#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <orb/orbit.h>
#include <ctype.h>
#include "echo.h"
#include "echo-share.h"

/**
	This is used by echo-server.c and echo-local.c
	It uses echo-skels.c
**/

static CORBA_ORB		the_orb;
static PortableServer_POA	the_poa;
static int                      the_i;

static CORBA_Object
do_echoString(PortableServer_Servant servant,
	      const CORBA_char *astring,
	      CORBA_double *outnum,
	      CORBA_Environment *ev)
{
    the_i = rand() % 100;
    *outnum = the_i;
    
    {
  	PortableServer_POA myCurPoa;
  	PortableServer_ObjectId *myCurOid;
	CORBA_char *myCurOidStr;
	PortableServer_Current poacur = (PortableServer_Current)
	    CORBA_ORB_resolve_initial_references(the_orb, "POACurrent", ev);
  	g_assert( ev->_major == 0 );
  	myCurPoa = PortableServer_Current_get_POA(poacur,ev);
  	g_assert( ev->_major == 0 );
  	myCurOid = PortableServer_Current_get_object_id(poacur,ev);
  	g_assert( ev->_major == 0 );
	
	g_assert( myCurPoa == the_poa );
	myCurOidStr = PortableServer_ObjectId_to_string(myCurOid, ev);
  	if ( !echo_opt_quiet )
      	    g_message("[server: oid=%s] %s ", myCurOidStr, astring);
	
  	CORBA_Object_release((CORBA_Object)myCurPoa,ev);
  	CORBA_free(myCurOid);
	CORBA_free(myCurOidStr);
  	CORBA_Object_release((CORBA_Object)poacur,ev);
    }
    
    return echo_srv_create_object(ev);
}

static void
do_doNothing(PortableServer_Servant servant, CORBA_Environment *ev) {
}

PortableServer_ServantBase__epv base_epv = {
  NULL,
  NULL,
  NULL
};
POA_Echo__epv echo_epv = { NULL, do_echoString, do_doNothing };
POA_Echo__vepv poa_echo_vepv = { &base_epv, &echo_epv };
POA_Echo poa_echo_servant = { NULL, &poa_echo_vepv };


void
echo_srv_start(CORBA_ORB orb, CORBA_Environment *ev)
{
    PortableServer_POAManager mgr;
    PortableServer_POA root_poa;
    CORBA_PolicyList policies;

    the_orb = orb;
    root_poa = 
	(PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, 
								 "RootPOA", 
								 ev);
    policies._maximum = 6;
    policies._length = 6;
    policies._buffer = g_new0(CORBA_Policy, 6);
    policies._buffer[0] = 
	(CORBA_Policy) PortableServer_POA_create_lifespan_policy(
            root_poa, 
            PortableServer_PERSISTENT, 
	    ev);
    policies._buffer[1] = 
	(CORBA_Policy) PortableServer_POA_create_id_assignment_policy(
            root_poa, 
	    PortableServer_USER_ID, 
	    ev);
    policies._buffer[2] = 
	(CORBA_Policy) PortableServer_POA_create_id_uniqueness_policy(
            root_poa,
	    PortableServer_MULTIPLE_ID,
	    ev);
    policies._buffer[3] = 
	(CORBA_Policy) PortableServer_POA_create_request_processing_policy(
            root_poa, 
	    PortableServer_USE_DEFAULT_SERVANT, 
	    ev);
    policies._buffer[4] = 
	(CORBA_Policy) PortableServer_POA_create_servant_retention_policy(
            root_poa,
	    PortableServer_NON_RETAIN,
	    ev);
    policies._buffer[5] = 
	(CORBA_Policy) ORBit_PortableServer_POA_create_okeyrand_policy(
            root_poa,
	    0, /* CORBA_short poa_rand_len */
	    0, /* CORBA_short obj_rand_len */
	    ev);
    g_assert(ev->_major == 0);
    mgr = PortableServer_POA__get_the_POAManager(root_poa, ev);
    the_poa = PortableServer_POA_create_POA(root_poa, "TheEchoPOA",
					    mgr, &policies, ev);
    if ( ev->_major ) {
	printf("create_POA failed: %d\n", ev->_major);
	exit(1);
    }
    PortableServer_POAManager_activate(mgr, ev);
    if ( ev->_major ) {
	printf("POAManager_activate failed: %d\n", ev->_major);
	exit(1);
    }
    CORBA_Object_release((CORBA_Object)mgr, ev);
    if ( ev->_major ) {
	printf("POAManager release failed: %d\n", ev->_major);
	exit(1);
    }
    CORBA_Object_release((CORBA_Object)root_poa, ev);
    if ( ev->_major ) {
	printf("root POA release failed: %d\n", ev->_major);
	exit(1);
    }
    POA_Echo__init(&poa_echo_servant, ev);
    if ( ev->_major ) {
	printf("object__init failed: %d\n", ev->_major);
	exit(1);
    }
    PortableServer_POA_set_servant(the_poa, &poa_echo_servant, ev);
    if ( ev->_major ) {
	printf("set_servant failed: %d\n", ev->_major);
	exit(1);
    }
}

CORBA_Object
echo_srv_create_object(CORBA_Environment *ev)
{
    PortableServer_ObjectId* object_id;
    Echo echo;
    static char oid_str[200];

    strcpy(oid_str, "Echo-");
    sprintf(oid_str + strlen(oid_str), "%d", the_i);
    object_id = PortableServer_string_to_ObjectId(oid_str, ev);
    if ( ev->_major ) {
	printf("string_to_ObjectId failed: %d\n", ev->_major);
	exit(1);
    }
    g_assert(object_id);
    g_printerr("new oid: `%s'\n", 
	       PortableServer_ObjectId_to_string(object_id, ev));
    if ( ev->_major ) {
	printf("ObjectId_to_string failed: %d\n", ev->_major);
	exit(1);
    }
    echo = PortableServer_POA_create_reference_with_id(the_poa, 
						       object_id, 
						       "IDL:Echo:1.0",
						       ev);
    if ( ev->_major ) {
	printf("create_reference_with_id failed: %d\n", ev->_major);
	exit(1);
    }
    CORBA_free(object_id);
    return echo;
}

void
echo_srv_finish(CORBA_Environment *ev)
{
    POA_Echo__fini(&poa_echo_servant, ev);
    if ( ev->_major ) {
	printf("object__fini failed: %d\n", ev->_major);
	exit(1);
    }
    CORBA_Object_release((CORBA_Object)the_poa, ev);
    if ( ev->_major ) {
	printf("POA release failed: %d\n", ev->_major);
	exit(1);
    }
    the_poa = 0;
}
